﻿using NetDisk;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;

namespace BaiduYunSync
{
    /// <summary>
    /// 同步类
    /// </summary>
    class Sync
    {

        static Sync()
        {
            Operation.InitUploadFile += Operation_InitUploadFile;
            Operation.StartUploadBlock += Operation_StartUploadBlock;
            Operation.ComplateUploadBlock += Operation_ComplateUploadBlock;
            Operation.UpdateComplate += Operation_UpdateComplate;
            Operation.UploadSpeed += Operation_UploadSpeed;
        }

        private static void Operation_UploadSpeed(string filePath, int fileBolckNum, double totalSecond, long offset, long blockTotalLenth)
        {
            //LogSettings.StdInfo(filePath + "已用时：" + totalSecond.ToString("F2") + "秒");
            //LogSettings.StdInfo(filePath + " 平均速度：" + (offset / 1024 / totalSecond).ToString("0.00") + "KB/秒");
            //LogSettings.StdInfo(filePath + "已上传：" + (offset * 100.0 / blockTotalLenth).ToString("F2") + "%");
        }

        private static void Operation_UpdateComplate(string filePath)
        {
            lock (locker)
            {
                LiteDbHelper.DeleteUpdateInfo(LiteDbHelper.GetAllUpdateInfos().Where(x => x.Path == filePath).First());
            }
            
            LogSettings.StdInfo(filePath + "上传完成，从队列中删除");
            if (StatusModify != null)
                StatusModify(filePath, StatusEnum.Delete);
            mre.Set();
        }

        private static void Operation_ComplateUploadBlock(string filePath, int endBlockNum, int totalBlockNum)
        {
            LiteDbHelper.GetAllUpdateInfos().Where(x => x.Path == filePath).ToList().ForEach(
                x =>
                {
                    x.ComplateBlockNum = endBlockNum;
                });
            LogSettings.StdInfo(string.Format("{0}完成第{1}个块，共{2}个块", filePath, endBlockNum, totalBlockNum));
            if (StatusModify != null)
                StatusModify(filePath, StatusEnum.Modify);
        }

        private static void Operation_StartUploadBlock(string filePath, int startBlockNum, int totalBlockNum)
        {
            LogSettings.StdInfo(string.Format("{0}开始上传第{1}个块，共{2}个块", filePath, startBlockNum, totalBlockNum));
        }

        private static void Operation_InitUploadFile(string filePath, int fileBlockNum, int needUploadBlockNum)
        {
            LiteDbHelper.GetAllUpdateInfos().Where(x => x.Path == filePath).ToList().ForEach(
                x => 
                {
                    x.Status = "正在上传";
                    x.TotalBlockNum = fileBlockNum;
                });
            LogSettings.StdInfo(filePath + "准备开始上传");
            if (StatusModify != null)
                StatusModify(filePath, StatusEnum.Start);
        }

        public static AutoResetEvent mre = new AutoResetEvent(false);

        /// <summary>
        /// 任务
        /// </summary>
        private static AutoResetEvent taskmre = new AutoResetEvent(false);


        private static object locker = new object();

        /// <summary>
        /// 开始新同步时发生
        /// </summary>
        /// <param name="folder">文件夹名称</param>
        public delegate void StartSyncHandle(string folder);

        public static event StartSyncHandle StartSync;

        /// <summary>
        /// 同步结束时发生
        /// </summary>
        /// <param name="folder"></param>
        public delegate void EndSyncHandle(SyncItemsDao folder);

        public static event EndSyncHandle EndSync;

        /// <summary>
        /// 文件上传状态改变
        /// </summary>
        /// <param name="path"></param>
        /// <param name="status"></param>
        public delegate void StatusModifyHandle(string path, StatusEnum status);

        public static event StatusModifyHandle StatusModify;

        /// <summary>
        /// 文件状态
        /// </summary>
        public enum StatusEnum
        {
            /// <summary>
            /// 添加新项
            /// </summary>
            Add,
            /// <summary>
            /// 开始上传
            /// </summary>
            Start,
            /// <summary>
            /// 上传状态改变
            /// </summary>
            Modify,
            /// <summary>
            /// 结束删除
            /// </summary>
            Delete
        }

        /// <summary>
        /// 启动同步线程
        /// </summary>
        /// <param name="credential">身份信息</param>
        public static void SyncNow( Credential credential)
        {
            // 启动5个上传线程进行上传
            for (int i = 0; i < 5; i++)
            {
                var thread = new Thread(() => UpdateFile(credential));
                thread.IsBackground = true;
                thread.Start();
            }
            while (true)
            {
                // 如果队列中存在未完成的任务，则不进行下一轮扫描
                while (LiteDbHelper.GetUpdateInfoCount() > 0)
                {
                    mre.WaitOne();
                }
                LogSettings.StdInfo("开始文件夹同步");
                // 查找距离上次同步时间超过6小时的文件夹
                var folderList = LiteDbHelper.GetAllSyncItems().Where(x => (DateTime.Now - x.LastSync).TotalHours > 6);
                foreach (var folder in folderList)
                {
                    LogSettings.StdInfo("需要同步的文件夹为：" + folder.Folder);
                    if (StartSync != null)
                    {
                        StartSync(folder.Folder);
                    }
                    SearchAllFile(folder.Folder, "/", credential);
                    folder.LastSync = DateTime.Now;
                    LiteDbHelper.UpdateSyncItem(folder);
                    LogSettings.StdInfo(folder.Folder + "同步完成");
                    if (EndSync != null)
                    {
                        EndSync(folder);
                    }
                }
                LogSettings.StdInfo("同步结束");
                mre.WaitOne(new TimeSpan(6, 0, 0));
            }
            
        }

        /// <summary>
        /// 查找所有需要同步的文件
        /// </summary>
        /// <param name="folderPath">本地文件夹路径</param>
        /// <param name="serverPath">服务器路径(父路径)</param>
        /// <param name="credential">身份信息</param>
        private static void SearchAllFile(string folderPath, string serverPath, Credential credential)
        {
            // 获取服务器路径下文件列表
            var fileList = Operation.GetFileList(serverPath, credential);
            // 获取需要同步的文件夹名
            var folderName = Path.GetFileName(folderPath);
            // 组合成新服务器路径，由于windows文件路径为\分割，服务器为/分割，所以需要转换
            var newServerPath = Path.Combine(serverPath, folderName).Replace('\\', '/');
            if (fileList.list == null || !fileList.list.Select(x => x.server_filename).Contains(folderName))
            {
                Operation.CreateFolder(newServerPath, credential);
                LogSettings.StdInfo("创建文件夹" + newServerPath);
            }
            fileList = Operation.GetFileList(newServerPath, credential);
            var localFileList = Directory.GetFiles(folderPath);
            var localDirectoryList = Directory.GetDirectories(folderPath);
            foreach (var file in localFileList)
            {
                // 如果任务中存在该文件则放弃添加
                if (LiteDbHelper.ContainsPath(file))
                {
                    continue;
                }
                var updateInfo = new UpdateInfoDao();
                updateInfo.FileName = Path.GetFileName(file);
                updateInfo.FileLength = new FileInfo(file).Length;
                updateInfo.Path = file;
                if (updateInfo.FileLength == 0)
                {
                    LogSettings.StdInfo(file + "大小为0，暂不支持上传");
                    continue;
                }
                var fileEnumberable = fileList.list == null ? null : fileList.list.Where(x => x.server_filename == Path.GetFileName(file));
                FileInfo fi = new FileInfo(file);
                //如果有文件，因为文件名不能相同，所以只需要判断第一个文件是否与服务器相同即可
                if (fileEnumberable != null && fileEnumberable.Count() > 0)
                {
                    // 服务器返回的MD5有时候好像有问题，会导致部分文件反复同步，所以删除
                    if (fileEnumberable.ElementAt(0).size == fi.Length)// && fileEnumberable.ElementAt(0).md5 == GetMD5HashFromFile(file))
                    {
                        LogSettings.StdInfo("文件已存在：" + fileEnumberable.ElementAt(0).server_filename);
                        continue;
                    }
                    updateInfo.NeedDelete = true;
                }
                if (!updateInfo.NeedDelete)
                {
                    updateInfo.ServerPath = Path.Combine(newServerPath, Path.GetFileName(file)).Replace('\\', '/');
                }
                lock (locker)
                {
                    LiteDbHelper.InsertUpdateInfo(updateInfo);
                    LogSettings.StdInfo("添加文件：" + updateInfo.Path);
                }
                if (StatusModify != null)
                    StatusModify(updateInfo.Path, StatusEnum.Add);
                taskmre.Set();
            }
            foreach (var directory in localDirectoryList)
            {
                SearchAllFile(directory, newServerPath, credential);
            }
        }

        /// <summary>
        /// 上传文件
        /// </summary>
        /// <param name="credential">身份信息</param>
        private static void UpdateFile(Credential credential)
        {
            while (true)
            {
                UpdateInfoDao updateInfo;
                lock (locker)
                {
                    // 查找第一个未开始同步的文件
                    updateInfo = LiteDbHelper.GetAllUpdateInfos().Where(x => !x.IsUsed).FirstOrDefault();
                    if (updateInfo != null)
                    {
                        updateInfo.IsUsed = true;
                    }
                        
                }
                // 如果没有找到，则证明
                if (updateInfo == null)
                {
                    taskmre.Reset();
                    taskmre.WaitOne();
                    continue;
                }
                if (updateInfo.NeedDelete)
                {
                    var fileOperationResult = Operation.Delete(updateInfo.ServerPath, credential);
                    if (!fileOperationResult.success)
                    {

                    }
                    LogSettings.StdInfo("删除文件：" + updateInfo.ServerPath);
                }
                for (int i = 0; i < 3; i++)
                {
                    try
                    {
                        Operation.ChunkedUpload(updateInfo.Path, updateInfo.ServerPath, credential);
                        LogSettings.StdInfo("上传文件成功：" + updateInfo.FileName);
                        break;
                    }
                    catch (Exception ex)
                    {
                        LogSettings.StdInfo(string.Format("第{0}次上传失败，文件名：{1}，失败原因：{2}", i, updateInfo.Path, ex));
                    }
                }
            }
            
        }
    }
}
