﻿using NetDisk;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace BaiduYunSync
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            //var checkResult = Authentication.LoginCheck("2000208");
            //MessageBox.Show(checkResult.needVCode.ToString());
            if (!SetQuota())
            {
                var login = new frmLogin();
                if (login.ShowDialog() == DialogResult.OK)
                {
                    if (!SetQuota())
                    {
                        Application.Exit();
                    }
                }
                else
                {
                    //如果不是点击了确认，强行退出程序，使用Application.Exit()会导致子线程报错
                    System.Environment.Exit(0);
                }
            }
            var items = LiteDbHelper.GetAllSyncItems();
            foreach (var item in items)
            {
                addListViewItem(item);
            }
            Sync.StartSync += Sync_StartSync;
            Sync.EndSync += Sync_EndSync;
            Thread thread = new Thread(() => { Sync.SyncNow(Credential.Deserialize(LiteDbHelper.UserInfo.UserCredentialStr)); });
            thread.IsBackground = true;
            thread.Start();
        }

        private void Sync_EndSync(SyncItemsDao folder)
        {
            lvSyncFolder.BeginInvoke(new Action(
            () =>
            {
                var items = lvSyncFolder.Items.Cast<ListViewItem>().Where(x => x.Text == folder.Folder);
                if (items.Count() > 0)
                {
                    items.ElementAt(0).SubItems[1].Text = "分析完成";
                    items.ElementAt(0).SubItems[2].Text = folder.LastSync.ToString();
                }
            }
            ), null);
        }

        private void Sync_StartSync(string folder)
        {
            lvSyncFolder.BeginInvoke(new Action(
            () =>
            {
                var items = lvSyncFolder.Items.Cast<ListViewItem>().Where(x => x.Text == folder);
                if (items.Count() > 0)
                {
                    items.ElementAt(0).SubItems[1].Text = "开始分析";
                }
            }
            ), null);
        }

        private bool SetQuota()
        {
            var userInfo = LiteDbHelper.UserInfo;
            if (userInfo == null || userInfo.UserCredentialStr == null)
            {
                return false;
            }
            var quotaResult = Operation.GetQuota(Credential.Deserialize(userInfo.UserCredentialStr));
            if (!quotaResult.success)
            {
                MessageBox.Show("Error:" + quotaResult.exception);
                return false;
            }
            else if (quotaResult.errno != 0)
            {
                return false;
            }
            tsslQuota.Text = "空间使用：已用" + GetGB(quotaResult.used) + "/总共" + GetGB(quotaResult.total);
            return true;
        }

        /// <summary>
        /// 将B转换为GB
        /// </summary>
        /// <param name="b"></param>
        /// <returns></returns>
        private string GetGB(long b)
        {
            var bb = (double)b;
            for (int i = 0; i < 3; i++)
            {
                bb /= 1024;
            }
            return bb.ToString("f2") + "GB";
        }

        private void btnAddSync_Click(object sender, EventArgs e)
        {
            if (fbdSyncFloder.ShowDialog() == DialogResult.OK)
            {
                if (lvSyncFolder.Items.Cast<ListViewItem>().Select(x => (x.Tag as SyncItemsDao).Folder).Contains(fbdSyncFloder.SelectedPath))
                {
                    MessageBox.Show("当前文件夹已在同步列表中，无需再次添加");
                    return;
                }
                var dao = new SyncItemsDao();
                dao.Folder = fbdSyncFloder.SelectedPath;
                dao.LastSync = new DateTime(1970, 1, 1);
                LiteDbHelper.UpdateSyncItem(dao);
                addListViewItem(dao);
                Sync.mre.Set();
            }
        }

        private void addListViewItem(SyncItemsDao dao)
        {
            var item = new ListViewItem(dao.Folder);
            item.SubItems.Add("准备同步");
            item.SubItems.Add(dao.LastSync.ToString());
            item.Tag = dao;
            item.ToolTipText = dao.Folder;
            lvSyncFolder.Items.Add(item);
        }

        private void txtReadLog_Click(object sender, EventArgs e)
        {
            new frmReadLog().ShowDialog();
        }

        private void tsslQuota_Click(object sender, EventArgs e)
        {
            SetQuota();
        }

        private void btnDelSync_Click(object sender, EventArgs e)
        {
            foreach (ListViewItem item in lvSyncFolder.SelectedItems)
            {
                LiteDbHelper.DeleteSyncItem(item.Tag as SyncItemsDao);
                lvSyncFolder.Items.Remove(item);
            }
        }

        frmUploadList updateListForm;
        private void btnUploadList_Click(object sender, EventArgs e)
        {
            if (updateListForm == null || updateListForm.IsDisposed)
            {
                updateListForm = new frmUploadList();
            }
            updateListForm.Show();
        }
    }  
}
