﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace BaiduYunSync
{
    public partial class frmUploadList : Form
    {
        public frmUploadList()
        {
            InitializeComponent();
        }

        private void modifyTitle(int listCount, int totalCount)
        {
            Text = string.Format("传输文件列表：当前显示{0}条/共{1}条", listCount, totalCount);
        }

        /// <summary>
        /// 将B转换为MB
        /// </summary>
        /// <param name="b"></param>
        /// <returns></returns>
        private string getMB(long b)
        {
            var bb = (double)b;
            for (int i = 0; i < 2; i++)
            {
                bb /= 1024;
            }
            return bb.ToString("f2") + "MB";
        }

        private void initList()
        {
            lvList.Items.Clear();
            var updateInfoList = LiteDbHelper.GetAllUpdateInfos();
            var count = updateInfoList.Length > 99 ? 99 : updateInfoList.Length;
            for (int i = 0; i < count; i++)
            {
                var updateInfo = updateInfoList[i];
                addItem(updateInfo);
            }
            modifyTitle(count, updateInfoList.Length);
        }

        private void addItem(UpdateInfoDao updateInfo, int position = -1)
        {
            ListViewItem item = new ListViewItem(updateInfo.FileName);
            item.Tag = updateInfo;
            item.ToolTipText = updateInfo.Path;
            item.SubItems.Add(updateInfo.Status == null ? "准备传输" : updateInfo.Status);
            item.SubItems.Add(getMB(updateInfo.FileLength));
            item.SubItems.Add(updateInfo.Status == null ? "" :
                string.Format("已完成{0}块/共{1}块", updateInfo.ComplateBlockNum, updateInfo.TotalBlockNum));
            item.SubItems.Add(updateInfo.Status == null ? "" :
                string.Format("已完成{0}/共{1}", updateInfo.ComplateBlockNum == updateInfo.TotalBlockNum ?
                getMB(updateInfo.FileLength) : updateInfo.ComplateBlockNum * 4 + "MB", getMB(updateInfo.FileLength)));
            item.SubItems.Add(updateInfo.Path);
            if (position == -1)
            {
                lvList.Items.Add(item);
            }
            else
            {
                lvList.Items.Insert(position, item);
            }
        }

        private void removeItem(string path)
        {
            if (lvList.Items.Cast<ListViewItem>().Select(x => x.SubItems[5].Text).Contains(path))
            {
                lvList.Items.Cast<ListViewItem>().Where(x => x.SubItems[5].Text == path).First().Remove();
            }
        }

        private void modifyItem(UpdateInfoDao updateInfo)
        {
            if (lvList.Items.Cast<ListViewItem>().Select(x => x.SubItems[5].Text).Contains(updateInfo.Path))
            {
                var item = lvList.Items.Cast<ListViewItem>().Where(x => x.SubItems[5].Text == updateInfo.Path).First();
                item.SubItems[1].Text = updateInfo.Status;
                item.SubItems[3].Text = updateInfo.Status == null ? "" :
                string.Format("已完成{0}块/共{1}块", updateInfo.ComplateBlockNum, updateInfo.TotalBlockNum);
                item.SubItems[4].Text = updateInfo.Status == null ? "" :
                string.Format("已完成{0}/共{1}", updateInfo.ComplateBlockNum == updateInfo.TotalBlockNum ? 
                getMB(updateInfo.FileLength) : updateInfo.ComplateBlockNum * 4 + "MB", getMB(updateInfo.FileLength));
                item.Tag = updateInfo;
            }
        }

        private void frmUploadList_Load(object sender, EventArgs e)
        {
            initList();
            Sync.StatusModify += Sync_StatusModify;
        }

        private void Sync_StatusModify(string path, Sync.StatusEnum status)
        {
            lvList.BeginInvoke(new Action(
                () =>
                {
                    UpdateInfoDao updateInfo;
                    switch (status)
                    {
                        case Sync.StatusEnum.Add:
                            if (lvList.Items.Count < 99)
                            {
                                updateInfo = LiteDbHelper.GetUpdateInfoFormPath(path);
                                if (updateInfo != null)
                                {
                                    addItem(LiteDbHelper.GetUpdateInfoFormPath(path));
                                }

                            }
                            break;
                        case Sync.StatusEnum.Start:
                        case Sync.StatusEnum.Modify:
                            updateInfo = LiteDbHelper.GetUpdateInfoFormPath(path);
                            if (updateInfo != null)
                            {
                                modifyItem(updateInfo);
                            }

                            break;
                        case Sync.StatusEnum.Delete:
                            removeItem(path);
                            var count = LiteDbHelper.GetUpdateInfoCount();
                            if (count > 98)
                            {
                                addItem(LiteDbHelper.GetAllUpdateInfos()[98]);
                            }
                            modifyTitle(count > 99 ? 99 : lvList.Items.Count, count);
                            break;
                        default:
                            break;
                    }
                }
                ), null);
            
        }

        private void 删除选中任务ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            foreach (ListViewItem item in lvList.SelectedItems)
            {
                var updateInfo = item.Tag as UpdateInfoDao;
                if (updateInfo.Status != null)
                {
                    MessageBox.Show(updateInfo.FileName + "正在传输，暂不支持移除", "提示");
                    return;
                }
                LiteDbHelper.DeleteUpdateInfo(updateInfo);
            }
            initList();
            if (lvList.Items.Count == 0)
            {
                Sync.mre.Set();
            }
        }

        private void frmUploadList_FormClosing(object sender, FormClosingEventArgs e)
        {
            Sync.StatusModify -= Sync_StatusModify;
        }
    }
}
